@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
  */
 class CWidgetFormTrigOver extends CWidgetForm {
 
-	public function __construct($data) {
-		parent::__construct($data, WIDGET_TRIG_OVER);
+	public function __construct($data, $templateid) {
+		parent::__construct($data, $templateid, WIDGET_TRIG_OVER);
+
+		$this->data = self::convertDottedKeys($this->data);
 
 		// Output information option.
 		$field_show = (new CWidgetFieldRadioButtonList('show', _('Show'), [
@@ -51,14 +53,38 @@ class CWidgetFormTrigOver extends CWidgetForm {
 
 		$this->fields[$field_groups->getName()] = $field_groups;
 
-		// Application field.
-		$field_application = new CWidgetFieldApplication('application', _('Application'));
+		// Hosts.
+		$field_hosts = new CWidgetFieldMsHost('hostids', _('Hosts'));
+		$field_hosts->filter_preselect_host_group_field = 'groupids_';
 
-		if (array_key_exists('application', $this->data)) {
-			$field_application->setValue($this->data['application']);
+		if (array_key_exists('hostids', $this->data)) {
+			$field_hosts->setValue($this->data['hostids']);
 		}
 
-		$this->fields[$field_application->getName()] = $field_application;
+		$this->fields[$field_hosts->getName()] = $field_hosts;
+
+		// Tag evaltype (And/Or).
+		$field_evaltype = (new CWidgetFieldRadioButtonList('evaltype', _('Tags'), [
+			TAG_EVAL_TYPE_AND_OR => _('And/Or'),
+			TAG_EVAL_TYPE_OR => _('Or')
+		]))
+			->setDefault(TAG_EVAL_TYPE_AND_OR)
+			->setModern(true);
+
+		if (array_key_exists('evaltype', $this->data)) {
+			$field_evaltype->setValue($this->data['evaltype']);
+		}
+
+		$this->fields[$field_evaltype->getName()] = $field_evaltype;
+
+		// Tags array: tag, operator and value. No label, because it belongs to previous group.
+		$field_tags = new CWidgetFieldTags('tags', '');
+
+		if (array_key_exists('tags', $this->data)) {
+			$field_tags->setValue($this->data['tags']);
+		}
+
+		$this->fields[$field_tags->getName()] = $field_tags;
 
 		// Show suppressed problems.
 		$field_show_suppressed = (new CWidgetFieldCheckBox('show_suppressed', _('Show suppressed problems')))
