@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,6 +18,11 @@
 **/
 
 
+// Global constants.
+
+// Sync with SASS variable: $ui-transition-duration.
+const UI_TRANSITION_DURATION = 300;
+
 // Array indexOf method for javascript<1.6 compatibility
 if (!Array.prototype.indexOf) {
 	Array.prototype.indexOf = function (searchElement) {
@@ -71,10 +76,11 @@ var PageRefresh = {
 			return false;
 		}
 
-		this.delayLeft -= 1000;
-		if (this.delayLeft < 0) {
-			if (IE || ED) {
-				sessionStorage.scrollTop = jQuery(window).scrollTop();
+		this.delayLeft = Math.max(-1, this.delayLeft - 1000);
+
+		if (this.delayLeft < 0 && !overlays_stack.length) {
+			if (ED) {
+				sessionStorage.scrollTop = $('.wrapper').scrollTop();
 			}
 
 			location.reload();
@@ -103,94 +109,7 @@ var PageRefresh = {
 };
 
 /*
- * Main menu
- */
-var MMenu = {
-	menus:			{'view': 0, 'cm': 0, 'reports': 0, 'config': 0, 'admin': 0},
-	def_label:		null,
-	sub_active: 	false,
-	timeout_reset:	null,
-	timeout_change:	null,
-
-	init: function() {
-		// Detects when none of the selected elements are focused.
-		var elems = jQuery('.top-nav a, .top-subnav a').on('keydown', function(event) {
-			clearTimeout(this.timeout_reset);
-
-			if (event.which == 9) {
-				setTimeout(function() {
-					if (elems.toArray().indexOf(document.querySelector(':focus')) == -1) {
-						clearTimeout(this.timeout_reset);
-						this.timeout_reset = setTimeout(function() {
-							if (elems.toArray().indexOf(document.querySelector(':focus')) == -1){
-								MMenu.showSubMenu(MMenu.def_label)
-							}
-						}, 2500);
-					}
-				});
-			}
-		});
-	},
-
-	mouseOver: function(show_label) {
-		clearTimeout(this.timeout_reset);
-		this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '", true)', 10);
-		PageRefresh.restart();
-	},
-
-	keyUp: function(show_label, event) {
-		if (event.which == 13) {
-			clearTimeout(this.timeout_reset);
-			this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '", true)', 10);
-			PageRefresh.restart();
-		}
-	},
-
-	submenu_mouseOver: function() {
-		clearTimeout(this.timeout_reset);
-		clearTimeout(this.timeout_change);
-		PageRefresh.restart();
-	},
-
-	mouseOut: function() {
-		clearTimeout(this.timeout_change);
-		this.timeout_reset = setTimeout('MMenu.showSubMenu("' + this.def_label + '")', 2500);
-	},
-
-	showSubMenu: function(show_label, focus_subitem) {
-		var sub_menu = $('sub_' + show_label),
-			focus_subitem = focus_subitem || false;
-
-		if (sub_menu !== null) {
-			$(show_label).className = 'selected';
-			sub_menu.show();
-
-			if (focus_subitem) {
-				jQuery('li:first > a', sub_menu).focus();
-			}
-
-			for (var key in this.menus) {
-				if (key == show_label) {
-					continue;
-				}
-
-				var menu_cell = $(key);
-				if (menu_cell !== null) {
-					menu_cell.className = '';
-					jQuery('a', menu_cell).blur();
-				}
-
-				var sub_menu_cell = $('sub_' + key);
-				if (sub_menu_cell !== null) {
-					sub_menu_cell.hide();
-				}
-			}
-		}
-	}
-};
-
-/*
- * Audio control system
+ * Audio control system.
  */
 var AudioControl = {
 
@@ -213,36 +132,26 @@ var AudioControl = {
 	playOnce: function(name) {
 		this.stop();
 
-		if (IE) {
-			this.create(name, false);
+		var obj = jQuery('#audio');
+
+		if (obj.length > 0 && obj.data('name') === name) {
+			obj.trigger('play');
 		}
 		else {
-			var obj = jQuery('#audio');
-
-			if (obj.length > 0 && obj.data('name') === name) {
-				obj.trigger('play');
-			}
-			else {
-				this.create(name, false);
-			}
+			this.create(name, false);
 		}
 	},
 
 	playLoop: function(name, delay) {
 		this.stop();
 
-		if (IE) {
-			this.create(name, true);
+		var obj = jQuery('#audio');
+
+		if (obj.length > 0 && obj.data('name') === name) {
+			obj.trigger('play');
 		}
 		else {
-			var obj = jQuery('#audio');
-
-			if (obj.length > 0 && obj.data('name') === name) {
-				obj.trigger('play');
-			}
-			else {
-				this.create(name, true);
-			}
+			this.create(name, true);
 		}
 
 		AudioControl.loop(new Date().getTime() + delay * 1000);
@@ -254,67 +163,29 @@ var AudioControl = {
 		if (obj !== null) {
 			clearTimeout(AudioControl.timeoutHandler);
 
-			if (IE) {
-				obj.setAttribute('loop', false);
-				obj.setAttribute('playcount', 0);
-
-				try {
-					obj.stop();
-				}
-				catch (e) {
-					setTimeout(
-						function() {
-							try {
-								document.getElementById('audio').stop();
-							}
-							catch (e) {
-							}
-						},
-						100
-					);
-				}
-			}
-			else {
-				jQuery(obj).trigger('pause');
-			}
+			jQuery(obj).trigger('pause');
 		}
 	},
 
 	create: function(name, loop) {
-		if (IE) {
-			jQuery('#audio').remove();
+		var obj = jQuery('#audio');
 
-			jQuery('body').append(jQuery('<embed>', {
+		if (obj.length == 0 || obj.data('name') !== name) {
+			obj.remove();
+
+			var audioOptions = {
 				id: 'audio',
 				'data-name': name,
 				src: 'audio/' + name,
-				enablejavascript: true,
-				autostart: true,
-				loop: true,
-				playcount: loop ? 9999999 : 1,
-				height: 0
-			}));
-		}
-		else {
-			var obj = jQuery('#audio');
+				preload: 'auto',
+				autoplay: true
+			};
 
-			if (obj.length == 0 || obj.data('name') !== name) {
-				obj.remove();
-
-				var audioOptions = {
-					id: 'audio',
-					'data-name': name,
-					src: 'audio/' + name,
-					preload: 'auto',
-					autoplay: true
-				};
-
-				if (loop) {
-					audioOptions.loop = true;
-				}
-
-				jQuery('body').append(jQuery('<audio>', audioOptions));
+			if (loop) {
+				audioOptions.loop = true;
 			}
+
+			jQuery('body').append(jQuery('<audio>', audioOptions));
 		}
 	}
 };
@@ -383,6 +254,9 @@ var jqBlink = {
  */
 var hintBox = {
 
+	preload_hint_timer: null,
+	show_hint_timer: null,
+
 	/**
 	 * Initialize hint box event handlers.
 	 *
@@ -391,55 +265,171 @@ var hintBox = {
 	 */
 	bindEvents: function () {
 		jQuery(document).on('keydown click mouseenter mouseleave', '[data-hintbox=1]', function (e) {
+			var $target = jQuery(this).hasClass('hint-item')
+				? jQuery(this).siblings('.main-hint')
+				: jQuery(this);
 
-			if (jQuery(this).hasClass('hint-item')) {
-				var target = jQuery(this).siblings('.main-hint');
-			}
-			else {
-				var target = jQuery(this);
+			if (e.type === 'keydown') {
+				if (e.which != 13) {
+					return;
+				}
+
+				var offset = $target.offset(),
+					w = jQuery(window);
+
+				// Emulate a click on the left middle point of the target.
+				e.clientX = offset.left - w.scrollLeft();
+				e.clientY = offset.top - w.scrollTop() + ($target.height() / 2);
+				e.preventDefault();
 			}
 
-			switch (e.type) {
-				case 'mouseenter':
-					hintBox.showHint(e, target[0], target.next('.hint-box').html(), target.data('hintbox-class'), false,
-						target.data('hintbox-style')
-					);
-					break;
+			if ($target.data('hintbox-preload') && $target.next('.hint-box').children().length == 0) {
+				clearTimeout(hintBox.preload_hint_timer);
 
-				case 'mouseleave':
-					hintBox.hideHint(target[0], false);
-					break;
+				// Manually trigger preloaderCloseHandler for the previous preloader.
+				if (jQuery('#hintbox-preloader').length) {
 
-				case 'keydown':
-					if (e.which == 13 && target.data('hintbox-static') == 1) {
-						var offset = target.offset(),
-							w = jQuery(window);
-						// Emulate click on left middle point of link.
-						e.clientX = offset.left - w.scrollLeft();
-						e.clientY = offset.top - w.scrollTop() + (target.height() / 2);
-						e.preventDefault();
-
-						hintBox.showStaticHint(e, target[0], target.data('hintbox-class'), false,
-							target.data('hintbox-style')
-						);
+					// Prevent loading restart on repetitive click and keydown events.
+					if (e.type === 'click' || e.type === 'keydown') {
+						return false;
 					}
-					break;
 
-				case 'click':
-					if (target.data('hintbox-static') == 1) {
-						hintBox.showStaticHint(e, target[0], target.data('hintbox-class'), false,
-							target.data('hintbox-style')
-						);
-					}
-					break;
+					jQuery(document).trigger('click');
+				}
+
+				if (e.type === 'mouseleave') {
+					$target.blur();
+
+					return false;
+				}
+
+				var preloadHintHandler = function() {
+					hintBox.preloadHint(e, $target);
+				}
+
+				if (e.type === 'mouseenter') {
+					hintBox.preload_hint_timer = setTimeout(preloadHintHandler, 400);
+				}
+				else {
+					preloadHintHandler();
+				}
+
+				return false;
 			}
+
+			hintBox.displayHint(e, $target, 400);
+
+			return false;
 		});
 	},
 
+	displayHint: function(e, $target, delay = 0) {
+		clearTimeout(hintBox.show_hint_timer);
+
+		switch (e.handleObj.origType) {
+			case 'mouseenter':
+				var showHintHandler = function() {
+					hintBox.showHint(e, $target[0], $target.next('.hint-box').html(), $target.data('hintbox-class'),
+						false, $target.data('hintbox-style')
+					);
+				}
+
+				if (delay > 0) {
+					hintBox.show_hint_timer = setTimeout(showHintHandler, delay);
+				}
+				else {
+					showHintHandler();
+				}
+				break;
+
+			case 'mouseleave':
+				hintBox.hideHint($target[0], false);
+				$target.blur();
+				break;
+
+			case 'keydown':
+			case 'click':
+				if ($target.data('hintbox-static') == 1) {
+					hintBox.showStaticHint(e, $target[0], $target.data('hintbox-class'), false,
+						$target.data('hintbox-style')
+					);
+				}
+				break;
+		}
+	},
+
+	preloadHint: function(e, $target) {
+		var url = new Curl('zabbix.php'),
+			data = $target.data('hintbox-preload');
+
+		url.setArgument('action', 'hint.box');
+		url.setArgument('type', data.type);
+
+		var xhr = jQuery.ajax({
+			url: url.getUrl(),
+			method: 'POST',
+			data: {
+				data: data.data
+			},
+			dataType: 'json'
+		});
+
+		var $preloader = hintBox.createPreloader();
+
+		var preloader_timer = setTimeout(function() {
+			$preloader.fadeIn(200);
+			hintBox.positionElement(e, $target[0], $preloader);
+		}, 500);
+
+		addToOverlaysStack($preloader.prop('id'), $target[0], 'preloader', xhr);
+
+		xhr.done(function(resp) {
+			clearTimeout(preloader_timer);
+			overlayPreloaderDestroy($preloader.prop('id'));
+
+			var $hint_box = $target.next('.hint-box').empty();
+
+			if (resp.messages) {
+				$hint_box.append(resp.messages);
+			}
+			if (resp.data) {
+				$hint_box.append(resp.data);
+			}
+
+			hintBox.displayHint(e, $target);
+		});
+
+		jQuery(document)
+			.off('click', hintBox.preloaderCloseHandler)
+			.on('click', {id: $preloader.prop('id')}, hintBox.preloaderCloseHandler);
+	},
+
+	/**
+	 * Create preloader elements for the hint box.
+	 */
+	createPreloader: function() {
+		return jQuery('<div>', {
+			'id': 'hintbox-preloader',
+			'class': 'is-loading hintbox-preloader'
+		})
+			.appendTo($('.wrapper'))
+			.on('click', function(e) {
+				e.stopPropagation();
+			})
+			.hide();
+	},
+
+	/**
+	 * Event handler for the preloader elements destroy.
+	 */
+	preloaderCloseHandler: function(event) {
+		overlayPreloaderDestroy(event.data.id);
+	},
+
 	createBox: function(e, target, hintText, className, isStatic, styles, appendTo) {
 		var hintboxid = hintBox.getUniqueId(),
 			box = jQuery('<div></div>', {'data-hintboxid': hintboxid}).addClass('overlay-dialogue'),
-			appendTo = appendTo || 'body';
+			appendTo = appendTo || '.wrapper';
 
 		if (styles) {
 			// property1: value1; property2: value2; property(n): value(n)
@@ -510,7 +500,7 @@ var hintBox = {
 
 			if (resizeAfterLoad) {
 				hintText.one('load', function(e) {
-					hintBox.positionHint(e, target);
+					hintBox.positionElement(e, target, target.hintBoxItem);
 				});
 			}
 		}
@@ -522,83 +512,75 @@ var hintBox = {
 		}
 
 		target.hintBoxItem = hintBox.createBox(e, target, hintText, className, isStatic, styles);
-		hintBox.positionHint(e, target);
+		hintBox.positionElement(e, target, target.hintBoxItem);
 		target.hintBoxItem.show();
 
 		if (target.isStatic) {
-			overlayDialogueOnLoad(true, target.hintBoxItem);
+			Overlay.prototype.recoverFocus.call({'$dialogue': target.hintBoxItem});
+			Overlay.prototype.containFocus.call({'$dialogue': target.hintBoxItem});
 		}
 	},
 
-	positionHint: function(e, target) {
-		var wWidth = jQuery(window).width(),
-			wHeight = jQuery(window).height(),
-			scrollTop = jQuery(window).scrollTop(),
-			scrollLeft = jQuery(window).scrollLeft(),
-			hint_width = jQuery(target.hintBoxItem).outerWidth(),
-			hint_height = jQuery(target.hintBoxItem).outerHeight(),
-			top, left;
-
-		// uses stored clientX on afterload positioning when there is no event
+	positionElement: function(e, target, $elem) {
 		if (e.clientX) {
 			target.clientX = e.clientX;
 			target.clientY = e.clientY;
 		}
 
-		// doesn't fit in the screen horizontally
-		if (hint_width + 10 > wWidth) {
-			left = scrollLeft + 2;
+		var $host = $elem.offsetParent(),
+			host_offset = $host.offset(),
+			// Usable area relative to host.
+			host_x_min = $host.scrollLeft(),
+			host_x_max = Math.min($host[0].scrollWidth,
+				$(window).width() + $(window).scrollLeft() - host_offset.left + $host.scrollLeft()
+			) - 1,
+			host_y_min = $host.scrollTop(),
+			host_y_max = Math.min($host[0].scrollHeight,
+				$(window).height() + $(window).scrollTop() - host_offset.top + $host.scrollTop()
+			) - 1,
+			// Event coordinates relative to host.
+			event_x = target.clientX - host_offset.left + $host.scrollLeft(),
+			event_y = target.clientY - host_offset.top + $host.scrollTop(),
+			event_offset = 10,
+			// Hint box width and height.
+			hint_width = $elem.outerWidth(),
+			hint_height = $elem.outerHeight(),
+			/*
+				Fix popup width and height since browsers will tend to reduce the size of the popup, if positioned further
+				than the width of window when horizontal scrolling is active.
+			*/
+			css = {
+				width: $elem.width(),
+				height: $elem.height()
+			};
+
+		if (event_x + event_offset + hint_width <= host_x_max) {
+			css.left = event_x + event_offset;
 		}
-		// 10px to right if fit
-		else if (wWidth - target.clientX - 10 > hint_width) {
-			left = scrollLeft + target.clientX + 10;
-		}
-		// 10px from screen right side
 		else {
-			left = scrollLeft + wWidth - 10 - hint_width;
+			css.right = -$host.scrollLeft();
 		}
 
-		// 10px below if fit
-		if (wHeight - target.clientY - hint_height - 10 > 0) {
-			top = scrollTop + target.clientY + 10;
+		if (event_y + event_offset + hint_height <= host_y_max) {
+			css.top = event_y + event_offset;
 		}
-		// 10px above if fit
-		else if (target.clientY - hint_height - 10 > 0) {
-			top = scrollTop + target.clientY - hint_height - 10;
+		else if (event_y - event_offset - hint_height >= host_y_min) {
+			css.top = event_y - event_offset - hint_height;
 		}
-		// 10px below as fallback
 		else {
-			top = scrollTop + target.clientY + 10;
-		}
+			css.top = Math.max(host_y_min, Math.min(host_y_max - hint_height, event_y + event_offset));
 
-		// fallback if doesn't fit vertically but could fit if aligned to right or left
-		if ((top - scrollTop + hint_height > wHeight)
-				&& (target.clientX - 10 > hint_width || wWidth - target.clientX - 10 > hint_width)) {
+			if (css.right !== undefined) {
+				delete css.right;
 
-			// align to left if fit
-			if (wWidth - target.clientX - 10 > hint_width) {
-				left = scrollLeft + target.clientX + 10;
-			}
-			// align to right
-			else {
-				left = scrollLeft + target.clientX - hint_width - 10;
-			}
-
-			// 10px from bottom if fit
-			if (wHeight - 10 > hint_height) {
-				top = scrollTop + wHeight - hint_height - 10;
-			}
-			// 10px from top
-			else {
-				top = scrollTop + 10;
+				css.left = ((event_x - event_offset - hint_width >= host_x_min)
+					? event_x - event_offset - hint_width
+					: event_x + event_offset
+				);
 			}
 		}
 
-		target.hintBoxItem.css({
-			top: top + 'px',
-			left: left + 'px',
-			zIndex: 1001
-		});
+		$elem.css(css);
 	},
 
 	hideHint: function(target, hideStatic) {
@@ -687,11 +669,11 @@ function changeWidgetState(obj, widgetId, idx) {
 
 	if (css === 'btn-widget-expand') {
 		jQuery('.body', widgetObj).slideUp(50);
-		jQuery('.dashbrd-widget-foot', widgetObj).slideUp(50);
+		jQuery('.dashboard-widget-foot', widgetObj).slideUp(50);
 	}
 	else {
 		jQuery('.body', widgetObj).slideDown(50);
-		jQuery('.dashbrd-widget-foot', widgetObj).slideDown(50);
+		jQuery('.dashboard-widget-foot', widgetObj).slideDown(50);
 
 		state = 1;
 	}
@@ -841,6 +823,7 @@ function getConditionFormula(conditions, evalType) {
 	 * - row					- element row selector
 	 * - add					- add row button selector
 	 * - remove					- remove row button selector
+	 * - rows					- array of rows objects data
 	 * - counter 				- number to start row enumeration from
 	 * - dataCallback			- function to generate the data passed to the template
 	 * - remove_next_sibling	- remove also next element
@@ -863,6 +846,7 @@ function getConditionFormula(conditions, evalType) {
 			disable: '.element-table-disable',
 			counter: null,
 			beforeRow: null,
+			rows: [],
 			dataCallback: function(data) {
 				return {};
 			}
@@ -900,9 +884,49 @@ function getConditionFormula(conditions, evalType) {
 				// disable the parent row
 				disableRow($(this).closest(options.row));
 			});
+
+			if (typeof options.rows === 'object') {
+				var before_row = (options['beforeRow'] !== null)
+					? $(options['beforeRow'], table)
+					: $(options.add, table).closest('tr');
+
+				initRows(table, before_row, options);
+			}
 		});
 	};
 
+	/**
+	 * Renders options.rows array as HTML rows during initialization.
+	 *
+	 * @param {jQuery} table       Table jquery node.
+	 * @param {jQuery} before_row  Rendered rows will be inserted before this node.
+	 * @param {object} options     Object with options.
+	 */
+	function initRows(table, before_row, options) {
+		var template = new Template($(options.template).html()),
+			counter = table.data('dynamicRows').counter,
+			$row;
+
+		options.rows.forEach((data) => {
+			data.rowNum = counter;
+			$row = $(template.evaluate($.extend(data, options.dataCallback(data))));
+
+			for (const name in data) {
+				// Set 'z-select' value.
+				$row.find('z-select[name$="[' + counter + '][' + name + ']"]').val(data[name]);
+
+				// Set 'radio' value.
+				$row.find('[type="radio"][name$="[' + counter + '][' + name + ']"][value="' + data[name] + '"]')
+					.attr('checked', 'checked');
+			}
+
+			before_row.before($row);
+			++counter;
+		});
+
+		table.data('dynamicRows').counter = counter;
+	}
+
 	/**
 	 * Adds a row before the given row.
 	 *
@@ -946,6 +970,7 @@ function getConditionFormula(conditions, evalType) {
 	 * @param {jQuery} row
 	 */
 	function disableRow(row) {
+		row.find('textarea').prop('readonly', true);
 		row.find('input').prop('readonly', true);
 		row.find('button').prop('disabled', true);
 	}
@@ -986,15 +1011,10 @@ jQuery(function ($) {
 
 				var css = {};
 
-				if (IE9) {
-					css['-ms-transform-origin'] = transform;
-				}
-				else {
-					css['transform-origin'] = transform;
-					css['-webkit-transform-origin'] = transform;
-					css['-moz-transform-origin'] = transform;
-					css['-o-transform-origin'] = transform;
-				}
+				css['transform-origin'] = transform;
+				css['-webkit-transform-origin'] = transform;
+				css['-moz-transform-origin'] = transform;
+				css['-o-transform-origin'] = transform;
 
 				var divInner = $('<div>', {
 					'class': 'vertical_rotation_inner'
@@ -1023,8 +1043,27 @@ jQuery(function ($) {
 		});
 	};
 
-	if ((IE || ED) && typeof sessionStorage.scrollTop !== 'undefined') {
-		$(window).scrollTop(sessionStorage.scrollTop);
+	if (ED && typeof sessionStorage.scrollTop !== 'undefined') {
+		$('.wrapper').scrollTop(sessionStorage.scrollTop);
 		sessionStorage.removeItem('scrollTop');
 	}
 });
+
+window.addEventListener('load', e => {
+
+	/**
+	 * SideBar initialization.
+	 */
+	const sidebar = document.querySelector('.sidebar');
+
+	if (sidebar !== null) {
+		ZABBIX.MenuMain = new CMenu(document.querySelector('.menu-main'));
+		ZABBIX.UserMain = new CMenu(document.querySelector('.menu-user'));
+
+		ZABBIX.Sidebar = new CSidebar(sidebar)
+			.on('viewmodechange', (e) => {
+				updateUserProfile('web.sidebar.mode', e.detail.view_mode, []);
+				window.dispatchEvent(new Event('resize'));
+			});
+	}
+});
